##############################################################################
#
# Copyright (c) 2010 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
import os

from slapos.recipe.librecipe import GenericBaseRecipe


def _mydumper_base_cmd(mydumper, database, user, password,
                       socket=None, host=None, port=None, **kw):
  cmd = [mydumper]
  cmd.extend(['-B', database])

  if socket:
    cmd.extend(['-S', socket])
  else:
    cmd.extend(['-h', host])
    cmd.extend(['-P', port])

  cmd.extend(['-u', user])
  if password:
    cmd.extend(['-p', password])

  return cmd


def do_export(args):
  cmd = _mydumper_base_cmd(**args)

  if args['compression']:
    cmd.append('--compress')

  if args['rows'] is not None:
    cmd.extend(['-r', args['rows']])

  cmd.extend(['-o', args['directory']])

  os.execv(cmd[0], cmd)


def do_import(args):
  cmd = _mydumper_base_cmd(**args)
  cmd.append('--overwrite-tables')
  cmd.extend(['-d', args['directory']])
  os.execv(cmd[0], cmd)


class Recipe(GenericBaseRecipe):

  def install(self):
    config = {
      'database': self.options['database'],
      'directory': self.options['backup-directory'],
      'user': self.options['user'],
      'password': self.options.get('password'),
    }

    if self.options.get('host'):
      config['host'] = self.options['host']
      config['port'] = self.options.get('port', 3306)
    elif self.options.get('socket'):
      config['socket'] = self.options['socket']
    else:
      raise ValueError("host or socket must be defined")

    if self.optionIsTrue('import', False):
      function = do_import
      config['mydumper'] = self.options['myloader-binary']
    else:
      function = do_export
      config['mydumper'] = self.options['mydumper-binary']
      config['compression'] = self.optionIsTrue('compression', default=False)
      config['rows'] = self.options.get('rows')

    return self.createPythonScript(self.options['wrapper'],
      '%s.%s' % (function.__module__, function.__name__), (config,))
